/* Copyright (c) 2001 Miller Puckette and others.
* For information on usage and redistribution, and for a DISCLAIMER OF ALL
* WARRANTIES, see the file, "LICENSE.txt," in this distribution.  */

/* this file calls Ross Bencina's and Phil Burk's Portaudio package.  It's
    the main way in for Mac OS and will later be loaded as a DLL into the
    NT version to offer directX there. */


#include "m_imp.h"
#include <stdio.h>
#include <stdlib.h>
#include "s_pa_pablio.h"

/* the public interface */

int pa_open_audio(int inchans, int outchans, int rate, t_sample *soundin,
    t_sample *soundout, int advance);
void pa_close_audio(void);
int pa_send_dacs(void);

static PABLIO_Stream  *pa_stream;
static int pa_inchans, pa_outchans;
static float *pa_soundin, *pa_soundout;
#define NCHANNELS 2
#define SAMPLES_PER_FRAME NCHANNELS * DACBLKSIZE

    /* so how do you set latency????*/
int pa_open_audio(int inchans, int outchans, int rate, t_sample *soundin,
    t_sample *soundout, int advance)
{
    PaError err;
    /* post("in %d out %d rate %d", inchans, outchans, rate); */
    if (inchans != 0 && inchans != 2)
    	error("portaudio: I only do stereo so far.");
    else if (outchans != 0 && outchans != 2)
    	error("portaudio: I only do stereo so far.");
    if (inchans && outchans)
        post("read/write"),
    	err = OpenAudioStream( &pa_stream, rate, paFloat32,
	    (PABLIO_READ_WRITE | PABLIO_STEREO) );
    else if (inchans)
    	err = OpenAudioStream( &pa_stream, rate, paFloat32,
	    (PABLIO_READ | PABLIO_STEREO) );
    else if (outchans)
    	err = OpenAudioStream( &pa_stream, rate, paFloat32,
	    (PABLIO_WRITE | PABLIO_STEREO) );
    else err = 0;
    if ( err != paNoError ) 
    {
	Pa_Terminate();
	fprintf( stderr, "Error number %d occured opening portaudio stream\n",
	    err); 
	fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );
	return (1);
    }
    pa_inchans = inchans;
    pa_outchans = outchans;
    pa_soundin = soundin;
    pa_soundout = soundout;
    return (0);
}

void pa_close_audio( void)
{
    if (pa_inchans || pa_outchans)
    	CloseAudioStream( pa_stream );
    pa_inchans = pa_outchans = 0;
}


int pa_send_dacs(void)
{
    float samples[SAMPLES_PER_FRAME], *fp1, *fp2;
    int i, j;
    double timebefore;
    
    timebefore = sys_getrealtime();
    if (pa_inchans)
    {
    	ReadAudioStream(pa_stream, samples, DACBLKSIZE);
    	for (j = 0, fp1 = pa_soundin; j < pa_inchans; j++, fp1 += DACBLKSIZE)
    	    for (i = 0, fp2 = samples + j; i < DACBLKSIZE; i++,
	    	fp2 += pa_inchans)
	{
	    fp1[i] = *fp2;
	}
    }
    if (pa_outchans)
    {
    	for (j = 0, fp1 = pa_soundout; j < pa_outchans; j++, fp1 += DACBLKSIZE)
    	    for (i = 0, fp2 = samples + j; i < DACBLKSIZE; i++,
	    	fp2 += pa_outchans)
	{
	    *fp2 = fp1[i];
            fp1[i] = 0;
	}
    	WriteAudioStream(pa_stream, samples, DACBLKSIZE);
    }

    if (sys_getrealtime() > timebefore + 0.002)
    	return (SENDDACS_SLEPT);
    else return (SENDDACS_YES);
}
